const htmlTagsRegexPattern = /<\s*[a-z]+\s*[^>]*>/
const NO_HTML_TAGS_ERROR_MESSAGE = 'Do not use html tags as key values'
const DUPLICATE_KEY_ERROR_MESSAGE = 'Duplicate key added. Kindly add a new key'
const DUPLICATE_VALUE_ADDED = 'The value already exists for another key. Use that key or add a different value'

module.exports ={
    meta :{
        type: 'problem',
        docs: {
            description: 'This rule validates a .properties against various validations'
        }
    },
    create: function(context){

        return {
            Program: function(){
                const filename = context.getFilename()
                const excludes = context.options[0]
                const minNoOfPartsInKey  = excludes.minNoOfPartsInKey
                const ignoreCheckComment = excludes.ignoreCheckComment.trim()
                const INCORRECT_KEY_LENGTH_ERROR_MESSAGE = `Key should contain a minimum of ${minNoOfPartsInKey} parts`

                if( !filename.endsWith('.properties')){
                    return 
                }
                let propertiesFileContent = context.getSourceCode().lines
                let keysInPropertiesFile = []
                let commentsLineNumbers = []

                propertiesFileContent.map( (line,index) => {
                    const currentLineNumber = index + 1
                    if(line.trim() !== ''){
                        if(line.trim().startsWith('#') && line.trim().includes(ignoreCheckComment)){
                            commentsLineNumbers.push(currentLineNumber)
                        }
                        if(line.indexOf('=') !== -1 ){
                            const equalToIndex = line.indexOf('=')
                            const key = line.substring(0, equalToIndex).trim()
                            const value = line.substring(equalToIndex +1).trim()
                            keysInPropertiesFile.push({key,value,lineNumberOfKey:currentLineNumber})

                            let isCommentAdded = commentsLineNumbers.some(lineNumber => currentLineNumber - lineNumber === 1)

                            //reporting error in values containing have html tags  
                            if(htmlTagsRegexPattern.test(value) && !isCommentAdded){
                                context.report({
                                    message:NO_HTML_TAGS_ERROR_MESSAGE,
                                    loc: {
                                        start:{
                                            line: currentLineNumber,
                                            column: equalToIndex + 1
                                        }
                                    }
                                })
                            }

                            //reporting errors on duplicate keys
                            let isDuplicateKeyAdded = keysInPropertiesFile.some(keyObject => keyObject.key === key && keyObject.lineNumberOfKey - index !== 1)
                            if(isDuplicateKeyAdded && !isCommentAdded){
                                context.report({
                                    message: DUPLICATE_KEY_ERROR_MESSAGE,
                                    loc:{
                                        start:{
                                            line: currentLineNumber,
                                            column: 0
                                        }
                                    }
                                })
                            }

                            //reporting errors on duplicate values
                            let isDuplicateValueAdded = keysInPropertiesFile.some(keyObject => keyObject.value === value && keyObject.lineNumberOfKey - index !== 1)
                            if(isDuplicateValueAdded && !isCommentAdded){
                                context.report({
                                    message: DUPLICATE_VALUE_ADDED,
                                    loc:{
                                        start:{
                                            line: currentLineNumber,
                                            column: equalToIndex + 1
                                        }
                                    }
                                })
                            }
                            
                            //reporting errors if keys doesn't contain a minimum number of parts
                            let noOfPartsInKey = 0
                            if(key.indexOf('.') !== -1 && key.indexOf('.') + 1 !== key.length){
                                noOfPartsInKey = key.split('.')
                            }
                            else if(key.indexOf('\\ ') !== -1){
                                noOfPartsInKey = key.split('\\ ')
                            }
                            let isInCorrectKey = noOfPartsInKey.length < minNoOfPartsInKey ? true : false
                            if(isInCorrectKey && !isCommentAdded){
                                context.report({
                                    message: INCORRECT_KEY_LENGTH_ERROR_MESSAGE,
                                    loc:{
                                        start:{
                                            line: currentLineNumber,
                                            column: 0
                                        }
                                    }
                                })
                            }
                        }
                    }   
                })
            }
        }
    }
}